@@ -1,15 +1,15 @@
-;;; haml-mode.el --- Major mode for editing Haml files
+;;; haml-mode.el --- Major mode for editing Haml files  -*- lexical-binding: t -*-
 
-;; Copyright (c) 2007, 2008 Nathan Weizenbaum
+;; Copyright (c) 2007, 2008 Natalie Weizenbaum
 
-;; Author: Nathan Weizenbaum
-;; URL: http://github.com/nex3/haml/tree/master
-;; Package-Requires: ((ruby-mode "1.0"))
-;; Package-Version: 3.1.9
-;; Package-Revision: 5e0baf7b795b
+;; Author: Natalie Weizenbaum
+;; URL: https://github.com/nex3/haml-mode
+;; Package-Requires: ((emacs "24.1") (cl-lib "0.5"))
+;; Package-Version: 20250714.1441
+;; Package-Revision: 3bb4a96535eb
 ;; Created: 2007-03-08
-;; By: Nathan Weizenbaum
-;; Keywords: markup, language, html
+;; By: Natalie Weizenbaum
+;; Keywords: markup, languages, html
 
 ;;; Commentary:
 
@@ -24,17 +24,14 @@
 
 ;;; Code:
 
-(eval-when-compile (require 'cl))
+(require 'cl-lib)
 (require 'ruby-mode)
 
 ;; Additional (optional) libraries for fontification
 (require 'css-mode nil t)
 (require 'textile-mode nil t)
 (require 'markdown-mode nil t)
-(or
- (require 'js nil t)
- (require 'javascript-mode "javascript" t))
-
+(require 'js nil t)
 
 ;; User definable variables
 
@@ -119,7 +116,7 @@ The line containing RE is matched, as well as all lines indented beneath it."
 ;; Fontifying sub-regions for other languages
 
 (defun haml-fontify-region
-  (beg end keywords syntax-table syntactic-keywords syntax-propertize-fn)
+    (beg end keywords syntax-table syntax-propertize-fn)
   "Fontify a region between BEG and END using another mode's fontification.
 
 KEYWORDS, SYNTAX-TABLE, SYNTACTIC-KEYWORDS and
@@ -131,7 +128,6 @@ respectively."
     (save-match-data
       (let ((font-lock-keywords keywords)
             (font-lock-syntax-table syntax-table)
-            (font-lock-syntactic-keywords syntactic-keywords)
             (syntax-propertize-function syntax-propertize-fn)
             (font-lock-multiline 'undecided)
             (font-lock-dont-widen t)
@@ -144,14 +140,16 @@ respectively."
           ;; so we have to move the beginning back one char
           (font-lock-fontify-region (1- beg) end))))))
 
+(defvar ruby-font-lock-syntax-table)
 (defun haml-fontify-region-as-ruby (beg end)
   "Use Ruby's font-lock variables to fontify the region between BEG and END."
   (haml-fontify-region beg end ruby-font-lock-keywords
-                       ruby-font-lock-syntax-table
-                       (when (boundp 'ruby-font-lock-syntactic-keywords)
-                         ruby-font-lock-syntactic-keywords)
-                       (when (fboundp 'ruby-syntax-propertize-function)
-                         #'ruby-syntax-propertize-function)))
+                       (if (boundp 'ruby-mode-syntax-table)
+                           ruby-mode-syntax-table
+                         ruby-font-lock-syntax-table)
+                       (if (fboundp 'ruby-syntax-propertize)
+                           'ruby-syntax-propertize
+                         'ruby-syntax-propertize-function)))
 
 (defun haml-fontify-region-as-css (beg end)
   "Fontify CSS code from BEG to END.
@@ -159,11 +157,10 @@ respectively."
 This requires that `css-mode' is available.
 `css-mode' is included with Emacs 23."
   (when (boundp 'css-font-lock-keywords)
-      (haml-fontify-region beg end
-                           css-font-lock-keywords
-                           css-mode-syntax-table
-                           nil
-                           nil)))
+    (haml-fontify-region beg end
+                         css-font-lock-keywords
+                         css-mode-syntax-table
+                         'css-syntax-propertize-function)))
 
 (defun haml-fontify-region-as-javascript (beg end)
   "Fontify javascript code from BEG to END.
@@ -171,16 +168,16 @@ This requires that `css-mode' is available.
 This requires that Karl Landström's javascript mode be available, either as the
 \"js.el\" bundled with Emacs >= 23, or as \"javascript.el\" found in ELPA and
 elsewhere."
-  (let ((keywords (or (and (featurep 'js) js--font-lock-keywords-3)
-                      (and (featurep 'javascript-mode) js-font-lock-keywords-3)))
-        (syntax-table (or (and (featurep 'js) js-mode-syntax-table)
-                          (and (featurep 'javascript-mode) javascript-mode-syntax-table)))
-        (syntax-propertize (and (featurep 'js) 'js-syntax-propertize)))
-    (when keywords
-      (when (and (fboundp 'js--update-quick-match-re) (null js--quick-match-re-func))
-        (js--update-quick-match-re))
-      (haml-fontify-region beg end keywords syntax-table nil syntax-propertize))))
+  (when js--font-lock-keywords-3
+    (when (and (fboundp 'js--update-quick-match-re)
+               (null js--quick-match-re-func))
+      (js--update-quick-match-re))
+    (haml-fontify-region beg end
+                         js--font-lock-keywords-3
+                         js-mode-syntax-table
+                         #'js-syntax-propertize)))
 
+(defvar textile-mode-syntax-table)
 (defun haml-fontify-region-as-textile (beg end)
   "Highlight textile from BEG to END.
 
@@ -190,8 +187,9 @@ Note that the results are not perfect, since `textile-mode' expects
 certain constructs such as \"h1.\" to be at the beginning of a line,
 and indented Haml filters always have leading whitespace."
   (if (boundp 'textile-font-lock-keywords)
-      (haml-fontify-region beg end textile-font-lock-keywords nil nil nil)))
+      (haml-fontify-region beg end textile-font-lock-keywords textile-mode-syntax-table nil)))
 
+(defvar markdown-mode-syntax-table)
 (defun haml-fontify-region-as-markdown (beg end)
   "Highlight markdown from BEG to END.
 
@@ -200,7 +198,6 @@ This requires that `markdown-mode' be available."
       (haml-fontify-region beg end
                            markdown-mode-font-lock-keywords
                            markdown-mode-syntax-table
-                           nil
                            nil)))
 
 (defvar haml-fontify-filter-functions-alist
@@ -262,25 +259,25 @@ This highlights the tag attributes and object refs of the tag,
 as well as the script expression (-, =, or ~) following the tag.
 
 For example, this will highlight all of the following:
-  %p{:foo => 'bar'}
+  %p{:foo => \='bar\='}
   %p[@bar]
-  %p= 'baz'
-  %p{:foo => 'bar'}[@bar]= 'baz'"
+  %p= \='baz\='
+  %p{:foo => \='bar'\=}[@bar]= \='baz\='"
   (when (re-search-forward "^[ \t]*[%.#]" limit t)
     (forward-char -1)
 
     ;; Highlight tag, classes, and ids
     (while (haml-move "\\([.#%]\\)[a-z0-9_:\\-]*")
       (put-text-property (match-beginning 0) (match-end 0) 'face
-                         (case (char-after (match-beginning 1))
+                         (cl-case (char-after (match-beginning 1))
                            (?% font-lock-keyword-face)
                            (?# font-lock-function-name-face)
                            (?. font-lock-variable-name-face))))
 
-    (block loop
+    (cl-block loop
       (while t
         (let ((eol (save-excursion (end-of-line) (point))))
-          (case (char-after)
+          (cl-case (char-after)
             ;; Highlight obj refs
             (?\[
              (forward-char 1)
@@ -293,7 +290,7 @@ For example, this will highlight all of the following:
              (while
                  (and (haml-parse-new-attr-hash
                        (lambda (type beg end)
-                         (case type
+                         (cl-case type
                            (name (put-text-property beg end
                                                     'face
                                                     font-lock-constant-face))
@@ -320,7 +317,7 @@ For example, this will highlight all of the following:
                      (haml-limited-forward-sexp eol))))
 
                (haml-fontify-region-as-ruby (1+ beg) (point))))
-            (t (return-from loop))))))
+            (t (cl-return-from loop))))))
 
     ;; Move past end chars
     (haml-move "[<>&!]+")
@@ -375,6 +372,9 @@ With ARG, do it that many times."
       (when start
         (cons start end)))))
 
+(defvar font-lock-beg)
+(defvar font-lock-end)
+
 (defun haml-maybe-extend-region (extender)
   "Maybe extend the font lock region using EXTENDER.
 With point at the beginning of the font lock region, EXTENDER is called.
@@ -485,11 +485,8 @@ changes in the initial region."
     (define-key map "\C-c\C-l" 'haml-output-buffer)
     map))
 
-(defalias 'haml-parent-mode
-  (if (fboundp 'prog-mode) 'prog-mode 'fundamental-mode))
-
 ;;;###autoload
-(define-derived-mode haml-mode haml-parent-mode "Haml"
+(define-derived-mode haml-mode prog-mode "Haml"
   "Major mode for editing Haml files.
 
 \\{haml-mode-map}"
@@ -570,9 +567,9 @@ either the end of the buffer or a line with no whitespace.
 If BACKWARD is non-nil, move the point backward instead."
   (let ((arg (if backward -1 1))
         (endp (if backward 'bobp 'eobp)))
-    (loop do (forward-line arg)
-          while (and (not (funcall endp))
-                     (looking-at "^[ \t]*$")))))
+    (cl-loop do (forward-line arg)
+             while (and (not (funcall endp))
+                        (looking-at "^[ \t]*$")))))
 
 (defun haml-at-indent-p ()
   "Return non-nil if the point is before any text on the line."
@@ -594,10 +591,10 @@ lines nested beneath it."
       (back-to-indentation)
     (while (/= arg 0)
       (let ((indent (current-indentation)))
-        (loop do (haml-forward-through-whitespace (< arg 0))
-              while (and (not (eobp))
-                         (not (bobp))
-                         (> (current-indentation) indent)))
+        (cl-loop do (haml-forward-through-whitespace (< arg 0))
+                 while (and (not (eobp))
+                            (not (bobp))
+                            (> (current-indentation) indent)))
         (unless (eobp)
           (back-to-indentation))
         (setq arg (+ arg (if (> arg 0) -1 1)))))))
@@ -619,9 +616,9 @@ With ARG, do this that many times."
   (or arg (setq arg 1))
   (while (> arg 0)
     (let ((indent (current-indentation)))
-      (loop do (haml-forward-through-whitespace t)
-            while (and (not (bobp))
-                       (>= (current-indentation) indent)))
+      (cl-loop do (haml-forward-through-whitespace t)
+               while (and (not (bobp))
+                          (>= (current-indentation) indent)))
       (setq arg (1- arg))))
   (back-to-indentation))
 
@@ -687,13 +684,13 @@ See http://www.w3.org/TR/html-markup/syntax.html.")
             (cdr (assq 'hash-indent attr-props))
           (+ (cdr (assq 'indent attr-props)) haml-indent-offset))
       (unless (or (haml-unnestable-tag-p))
-        (loop for opener in haml-block-openers
-              if (looking-at opener) return t
-              finally return nil)))))
+        (cl-loop for opener in haml-block-openers
+                 if (looking-at opener) return t
+                 finally return nil)))))
 
-(defun* haml-parse-multiline-attr-hash ()
-  "Parses a multiline attribute hash, and returns
-an alist with the following keys:
+(cl-defun haml-parse-multiline-attr-hash ()
+  "Parse a multiline attribute hash.
+Return value is an alist with the following keys:
 
 INDENT is the indentation of the line beginning the hash.
 
@@ -709,42 +706,42 @@ beginning the hash."
           (progn
             (goto-char (1- (match-end 0)))
             (haml-limited-forward-sexp (save-excursion (end-of-line) (point)))
-            (return-from haml-parse-multiline-attr-hash
+            (cl-return-from haml-parse-multiline-attr-hash
               (when (or (string-equal (match-string 1) "(") (eq (char-before) ?,))
                 `((indent . ,(current-indentation))
                   (hash-indent . ,(- (match-end 0) (match-beginning 0)))
                   (point . ,(match-beginning 0))))))
-        (when (bobp) (return-from haml-parse-multiline-attr-hash))
+        (when (bobp) (cl-return-from haml-parse-multiline-attr-hash))
         (forward-line -1)
         (unless (haml-unclosed-attr-hash-p)
-          (return-from haml-parse-multiline-attr-hash))))))
+          (cl-return-from haml-parse-multiline-attr-hash))))))
 
-(defun* haml-unclosed-attr-hash-p ()
+(cl-defun haml-unclosed-attr-hash-p ()
   "Return t if this line has an unclosed attribute hash, new or old."
   (save-excursion
     (end-of-line)
-    (when (eq (char-before) ?,) (return-from haml-unclosed-attr-hash-p t))
+    (when (eq (char-before) ?,) (cl-return-from haml-unclosed-attr-hash-p t))
     (re-search-backward "(\\|^")
     (haml-move "(")
     (haml-parse-new-attr-hash)))
 
-(defun* haml-parse-new-attr-hash (&optional (fn (lambda (type beg end) ())))
-  "Parse a new-style attribute hash on this line, and returns
-t if it's not finished on the current line.
+(cl-defun haml-parse-new-attr-hash (&optional (fn (lambda (_type _beg _end) ())))
+  "Parse a new-style attribute hash on this line.
+Return value is t if it's not finished on the current line.
 
 FN should take three parameters: TYPE, BEG, and END.
-TYPE is the type of text parsed ('name or 'value)
+TYPE is the type of text parsed (\='name or \='value)
 and BEG and END delimit that text in the buffer."
   (let ((eol (save-excursion (end-of-line) (point))))
     (while (not (haml-move ")"))
       (haml-move "[ \t]*")
       (unless (haml-move "[a-z0-9_:\\-]+")
-        (return-from haml-parse-new-attr-hash (haml-move "[ \t]*$")))
+        (cl-return-from haml-parse-new-attr-hash (haml-move "[ \t]*$")))
       (funcall fn 'name (match-beginning 0) (match-end 0))
       (haml-move "[ \t]*")
       (when (haml-move "=")
         (haml-move "[ \t]*")
-        (unless (looking-at "[\"'@a-z0-9]") (return-from haml-parse-new-attr-hash))
+        (unless (looking-at "[\"'@a-z0-9]") (cl-return-from haml-parse-new-attr-hash))
         (let ((beg (point)))
           (haml-limited-forward-sexp eol)
           (funcall fn 'value beg (point)))
@@ -778,10 +775,10 @@ between possible indentations."
     (setq end (point-marker))
     (goto-char start)
     (let (this-line-column current-column
-          (next-line-column
-           (if (and (equal last-command this-command) (/= (current-indentation) 0))
-               (* (/ (1- (current-indentation)) haml-indent-offset) haml-indent-offset)
-             (car (haml-compute-indentation)))))
+                           (next-line-column
+                            (if (and (equal last-command this-command) (/= (current-indentation) 0))
+                                (* (/ (1- (current-indentation)) haml-indent-offset) haml-indent-offset)
+                              (car (haml-compute-indentation)))))
       (while (< (point) end)
         (setq this-line-column next-line-column
               current-column (current-indentation))
@@ -789,8 +786,8 @@ between possible indentations."
         (delete-horizontal-space)
         (unless (eolp)
           (setq next-line-column (save-excursion
-                                   (loop do (forward-line 1)
-                                         while (and (not (eobp)) (looking-at "^[ \t]*$")))
+                                   (cl-loop do (forward-line 1)
+                                            while (and (not (eobp)) (looking-at "^[ \t]*$")))
                                    (+ this-line-column
                                       (- (current-indentation) current-column))))
           ;; Don't indent an empty line
@@ -806,8 +803,8 @@ back-dent the line by `haml-indent-offset' spaces.  On reaching column
 0, it will cycle back to the maximum sensible indentation."
   (interactive "*")
   (let ((ci (current-indentation))
-        (cc (current-column)))
-    (destructuring-bind (need strict) (haml-compute-indentation)
+        (_cc (current-column)))
+    (cl-destructuring-bind (need strict) (haml-compute-indentation)
       (save-excursion
         (beginning-of-line)
         (delete-horizontal-space)
@@ -850,7 +847,7 @@ the current line."
           (save-excursion
             (beginning-of-line)
             (looking-at "^[ \t]+$")))
-      (backward-delete-char arg)
+      (delete-char (- arg))
     (save-excursion
       (beginning-of-line)
       (unwind-protect
